Require emap.
Require Bool.
Require Sumbool.
Require Max.
Extraction Language Ocaml.

Extraction Inline proj1_sig.

(* physical equality *)
Extract Inlined Constant runtime.physical_equality => "( == )".

(* unit *)
Extract Inductive unit => unit [ "()" ].

(* option *) 
Extract Inductive option => option ["Some" "None"].

(* pair *)
Extract Inductive prod => "pair" [ "" ].

(* bool *)
Extract Inductive bool => "bool" [ "true" "false" ].
Extract Inlined Constant Bool.andb => "( && )".
Extract Inlined Constant Bool.orb => "( || )".

(* sumbool *)
Extract Inductive sumbool => "bool" [ "true" "false" ].
Extract Inlined Constant Sumbool.sumbool_and => "( && )".
Extract Inlined Constant Sumbool.sumbool_or => "( || )".
Extract Inlined Constant Sumbool.bool_of_sumbool => "(fun (x : bool) -> x)".
Extract Inlined Constant Sumbool.sumbool_of_bool => "(fun (x : bool) -> x)".

(* sumor *)
Extract Inductive sumor => "option" [ "Some" "None" ].

(* nat *)
Extract Inductive nat => "int" [ "0" "succ" ].
Extract Inlined Constant plus => "( + )".
Extract Inlined Constant minus => "( - )".
Extract Inlined Constant mult => "( * )".
Extract Inlined Constant pred => "pred".

Extract Constant nat_rect =>
  "(let rec recursor f f0 n =
    assert(n >= 0);
    if n = 0
    then f
    else let n0 = pred n in f0 n0 (recursor f f0 n0) in recursor)".
Extract Constant nat_rec => "nat_rect".

(* list *)
Require CoqList.
Extract Constant CoqList.nth =>
  "(let rec nth n l default =
    assert(n >= 0);
    match n with
    | 0 -> (match l with
            | Coq_nil -> default
            | Coq_cons (x, l') -> x)
    | _ -> (match l with
            | Coq_nil -> default
            | Coq_cons (x, t) -> nth (pred n) t default) in nth)".
Extract Constant CoqList.nth_ok =>
  "(let rec nth_ok n l default =
    assert(n >= 0);
    match n with
    | 0 -> (match l with
            | Coq_nil -> false
            | Coq_cons (x, l') -> true)
    | _ -> (match l with
            | Coq_nil -> false
            | Coq_cons (x, t) -> nth_ok (pred n) t default) in nth_ok)".
Extract Constant CoqList.nth_in_or_default =>
  "(let rec nth_in_or_default n l d =
  match l with
    | Coq_nil -> false
    | Coq_cons (a, l0) ->
        (match n with
           | 0 -> true
           | _ -> nth_in_or_default (pred n) l0 d) in  nth_in_or_default)".
Extract Constant CoqList.nth_error =>
  "(let rec nth_error l n =
  assert(n >= 0);
  match n with
  | 0 -> (match l with
            | Coq_nil -> None
            | Coq_cons (x, l0) -> value x)
  | _ ->
      (match l with
         | Coq_nil -> None
         | Coq_cons (a, l0) -> nth_error l0 (pred n)) in nth_error)".

(* Compare_dec *)
Extract Inlined Constant Compare_dec.zerop => "(( = ) 0)".
Extract Inlined Constant Compare_dec.le_lt_dec => "( <= )".
Extract Inlined Constant Compare_dec.le_gt_dec => "( <= )".
Extract Inlined Constant Compare_dec.lt_eq_lt_dec =>
  "(fun (x : int) (y : int) ->
      if x <= y then Some(x < y) else None)".
Extract Inlined Constant Compare_dec.le_lt_eq_dec => "( < )".

(* Max *)
Extract Inlined Constant Max.max => "( fun (x : int) (y : int) ->
  if (x > y) then x else y )".
Extract Inlined Constant Max.max_dec => "( >= )".
Extract Inlined Constant Max.max_case => "( fun (x : int) (y : int) xv yv ->
  if (x > y) then xv else yv )".

Recursive Extraction Library emap.
